function H = AndersonWhitenessTest(id_model, test_data, ...
                                    alpha_value, ...
                                    verboseFLAG)
% simple implementation of the Andreson's whiteness test 
% on 1-step-ahead prediction residual
% System Dynamics - 267MI - Fall 2018 - Univ. of Trieste
% G. Fenu
%   Detailed explanation goes here

if (alpha_value >0) && (alpha_value<1)
    alpha = alpha_value;
else
    error(['AndersonWhitenessTest: the chosen confidence level ', ...
           'must be strickly positive and below 1']);
end

prediction_residuals_iddataStruct = resid(test_data, id_model);
% evaluation of 1-step-ahead prediction residual using test_data and the
% model id_model: the result is an idData struct
prediction_residuals = prediction_residuals_iddataStruct.OutputData;
% the residuals

% how many data?
N = numel(test_data.InputData);
M = floor(N/4);
if verboseFLAG
    fprintf(1, ...
        'AndersonWhitenessTest: confidence level %f\n',...
        alpha);
    fprintf(1, ...
        'AndersonWhitenessTest: max lag value M %d\n',...
        M);
end
% the maximum value for lag, when evaluating 
% the sampled autocorrelation function (see SD L12 p30)
normAutocorrCoeffs = xcorr(prediction_residuals, ...
                            M, ...
                            'coeff');
% the normalized sampled autocorrelation function

% scaling the function values
scaled_hat_rho_coeffs = normAutocorrCoeffs * sqrt(N); 
% see SD L12 p30

% using the confidence level alpha, find the value beta such that
%               Prob{abs(x) > beta} = alpha
% where x is  gaussian r.v. with mean 0 and variance 1
beta = sqrt(2)*erfinv(alpha);
bag_of_values = find(abs(scaled_hat_rho_coeffs)<=beta);
P = numel(bag_of_values);
criticalValue = P/M;

if verboseFLAG
    fprintf(1, ...
        'AndersonWhitenessTest: critical value %f\n',...
        criticalValue);
end
if (criticalValue > alpha)
    H = 1;
    % refuse the hypothesis that scaled_hat_rho_coeffs is a r.v.
    % distributed as a G(0, 1)
    if verboseFLAG
        fprintf(1, ...
            ['AndersonWhitenessTest: refuse the hypothesis that \n',...
            'the prediction residuals are r.v. of a stationary \n',...
            'white noise stochastic process\n\n']);
    end
else
    H = 0;
    % accept the the hypothesis that scaled_hat_rho_coeffs is a r.v.
    % distributed as a G(0, 1)
       if verboseFLAG
        fprintf(1, ...
            ['AndersonWhitenessTest: accept the hypothesis that \n',...
            'the prediction residuals are r.v. of a stationary \n',...
            'white noise stochastic process\n\n']);
    end
end

end % function

